from json.decoder import JSONDecodeError
import logging

from evdev import InputDevice, categorize, ecodes
from evdev.events import InputEvent  # , KeyEvent
import requests

logger = logging.getLogger(__name__)


def handle_key_press(data: InputEvent) -> str:
    """Converts data.scancode to a digit.

    Digits: 0-9

    Args:
        data: InputEvent object representing a key press event.

    Returns:
        The last digit of the scancode.
    """
    digit = (data.scancode - 1) % 10
    return str(digit)


def send_post_request(
    session: requests.Session,
    url: str,
    tag_id: str,
    timeout: int = 5,
    ssl_verify: bool = False,
) -> dict:
    """Sends a POST request to the URL with the tag ID.

    Args:
        session: The requests.Session object.
        url: The URL to send the POST request.
        tag_id: The tag ID to send in the request body.
        ssl_verify: Whether to verify the SSL certificate.
            False allows to use self cert in local/internal network.

    Returns:
        A dictionary containing the` response data` and `status code`.
    """

    body = {"tag_id": tag_id}
    data = {"status_code": None, "response": None}
    try:
        response = session.post(url, json=body, timeout=timeout, verify=ssl_verify)
    except requests.exceptions.RequestException:
        logger.exception("Failed to send POST request")
        return
    else:
        data["status_code"] = response.status_code
    try:
        data["response"] = response.json()
    except JSONDecodeError:
        logger.exception("Error parsing response")
        return
    return data


def main_loop(reader: InputDevice, url: str, api_token: str, api_auth_scheme: str):
    """Main loop of the RFID reader.

    Args:
        reader: InputDevice object representing the RFID reader.
        url: The URL to send the POST request.
        api_token: The API token to send in the request headers.
        api_auth_scheme: The API authentication scheme to send in the request headers.
            example: Token (Basic, Bearer not supported)
    """
    rfid_tag = ""
    headers = {"Authorization": f"{api_auth_scheme} {api_token}"}
    session = requests.Session()
    session.headers.update(headers)
    logger.info(f"Listening for RFID scans on device: {reader.path}")

    # event: KeyEvent
    for event in reader.read_loop():
        if event.type != ecodes.EV_KEY:
            # logger.debug(f"Skipping non-key event: {event}")
            continue

        data: InputEvent = categorize(event)

        if data.keystate != 1:
            # logger.debug(f"Skipping non-press event: {data}")
            continue

        if 2 <= data.scancode <= 11:
            rfid_tag += handle_key_press(data)
        elif data.scancode == 28:
            logger.info(f"RFID Tag ID: {rfid_tag}")
            logger.info("Sending POST request")
            response_data = send_post_request(session, url, rfid_tag)
            logger.info(f"Response: {response_data}")
            rfid_tag = ""



import logging
import evdev

logger = logging.getLogger(__name__)


class DeviceInputError(Exception):
    pass


class DeviceInputNotFound(DeviceInputError):
    pass


def find_rfid_reader(device_name: str) -> evdev.InputDevice:
    """Attempt to locate an RFID reader device by a known identifier in its name.

    When input device is found then returns something like /dev/input/event8.

    Args:
        device_name: the exact name of the device.
            Use `ls /dev/input/by-id/` or `python -m evdev.evtest`.
            Example "Sycreader RFID Technology Co., Ltd SYC ID&IC USB Reader"

    Returns:
        RFID reader path of the input device or empty string if not found.

    Raises:
        DeviceInputNotFound:
    """
    devices = [evdev.InputDevice(path) for path in evdev.list_devices()]
    for device in devices:
        logger.debug(f"Checking device name: {device.name}, Path: {device.path}")
        if device_name in device.name:
            return device
    raise DeviceInputNotFound


if __name__ == "__main__":
    device_name = "Sycreader RFID Technology Co., Ltd SYC ID&IC USB Reader"
    rfid_reader_path = find_rfid_reader(device_name)

    if rfid_reader_path:
        # Use the found RFID reader path for further processing
        print(f"RFID Reader Path: {rfid_reader_path}")
    else:
        print("No RFID reader detected.")



import logging
import logging.handlers
from typing import List
import sys

FORMAT = "[%(asctime)s | %(module)s | %(levelname)s | %(lineno)d]: %(message)s"
DATETIME_FMT = "%Y-%m-%dT%H:%M:%S%z"


def logging_config(
    logger_level: str | int = logging.INFO,
    logger_fmt: str = FORMAT,
    logger_dt_fmt: str = DATETIME_FMT,
) -> None:
    """Default configuration logger.

    ..code::
        # main.py o any
        import logging
        ...
        import setup_logger # or the name of the this module
        ...
        setup_logger.logging_config()  # or the name of this function
        logger = logging.getLogger(__name__)
        ...

        # For any other module use the standar
        import logging
        ...
        logger = logging.getLogger(__name__)

    For other logger is possible to configure separete as
    setLevel or Propagate:

    .. code::
        logging.getLogger("pkg_name").setLevel("LEVEL")
        logging.getLogger("pkg_name").propagate = False
    """
    stream_handler = logging.StreamHandler(stream=sys.stdout)
    logging.basicConfig(
        level=logger_level,
        format=logger_fmt,
        datefmt=logger_dt_fmt,
        handlers=(stream_handler,),
    )


def disable_logger(loggers: List[str]) -> None:
    """Disable some loggers.

    logging.getLogger("urllib3").setLevel(logging.WARNING)
    """
    for logger in loggers:
        logging.getLogger(logger).setLevel(logging.WARNING)


def _show_loggers() -> None:
    """Print all loggers for entire application.

    Require import all modules to show all loggers available.

    ..code::

        import A
        import B
        ...

    Ref: https://stackoverflow.com/a/36208664/4112006
    """
    # import celery  # noqa: F401
    # import requests  # noqa: F401
    print("Available loggers:")
    for logger in logging.Logger.manager.loggerDict:
        print(logger)


def get_logger(logger_name, level: int = logging.INFO) -> logging.Logger:
    logging_config(level)
    return logging.getLogger(logger_name)


if __name__ == "__main__":
    _show_loggers()
